@@ -67,22 +67,15 @@ module Text.XML.HaXml.XmlContent.Parser
   , ANYContent(..)
   ) where
 
---import IO
-import Maybe (catMaybes)
-import Char  (chr, isSpace)
-import List  (isPrefixOf, isSuffixOf)
-
-import Text.PrettyPrint.HughesPJ (render)
-import qualified Text.XML.HaXml.Pretty as PP
+--import System.IO
+import Data.Maybe (catMaybes)
+import Data.Char  (chr, isSpace)
 
 import Text.XML.HaXml.Types
+import Text.XML.HaXml.Namespaces
 import Text.XML.HaXml.TypeMapping
-import Text.XML.HaXml.Posn     (Posn, posInNewCxt)
-import Text.XML.HaXml.Pretty   (document)
-import Text.XML.HaXml.Parse    (xmlParse)
+import Text.XML.HaXml.Posn     (Posn)
 import Text.XML.HaXml.Verbatim (Verbatim(verbatim))
-import qualified Text.XML.HaXml.ByteStringPP as FPS (document)
-import qualified Data.ByteString.Lazy.Char8 as FPS
 
 import Text.ParserCombinators.Poly
 
@@ -101,19 +94,19 @@ v `debug` _ = v
 ------------------------------------------------------------------------
 -- | Read a single attribute called "value".
 attval :: (Read a) => Element i -> a
-attval (Elem _ [("value",v@(AttValue _))] []) = read (show v)
+attval (Elem _ [(_{-N "value"-},v@(AttValue _))] []) = read (show v)
 
 -- | Generate a single attribute.
 mkAttr :: String -> String -> Attribute
-mkAttr n v = (n, AttValue [Left v])
+mkAttr n v = (N n, AttValue [Left v])
 
 -- | Generate an element with no attributes, named for its HType.
 mkElem :: XmlContent a => a -> [Content ()] -> Content ()
-mkElem x cs  = CElem (Elem (showHType (toHType x) "") [] cs) ()
+mkElem x cs  = CElem (Elem (N (showHType (toHType x) "")) [] cs) ()
 
 -- | Generate an element with no attributes, named directly.
 mkElemC :: String -> [Content ()] -> Content ()
-mkElemC x cs = CElem (Elem x [] cs) ()
+mkElemC x cs = CElem (Elem (N x) [] cs) ()
 
 -- | Turn a simple string into XML text.
 toText :: String -> [Content ()]
@@ -151,9 +144,10 @@ posnElementWith match tags = do
     { c <- content (formatted tags)
     ; case c of
           CElem e@(Elem t _ _) pos
-              | any (match t) tags -> return (pos, e)
-              | otherwise          -> fail ("Found a <"++t++">, but expected "
-                                           ++formatted tags++"\nat "++show pos)
+              | any (match (localName t)) tags -> return (pos, e)
+              | otherwise   -> fail ("Found a <"++printableName t
+                                     ++">, but expected "
+                                     ++formatted tags++"\nat "++show pos)
           CString b s pos
               | not b && all isSpace s -> posnElementWith match tags
                                                         -- ignore blank space
@@ -195,7 +189,8 @@ interior (Elem e _ cs) p =
         (Right x, []) -> return x
         (Right x, ds@(d:_))
             | all onlyMisc ds -> return x
-            | otherwise       -> fail ("Too many elements inside <"++e++"> at\n"
+            | otherwise       -> fail ("Too many elements inside <"
+                                      ++printableName e++"> at\n"
                                       ++show (info d)++"\n"
                                       ++"Found excess: "++verbatim d)
   where onlyMisc (CMisc _ _) = True
@@ -603,11 +598,11 @@ possibleA from at as = searchMaybe (from at) as
 
 fromAttrToStr :: String -> Attribute -> Maybe String
 fromAttrToStr n (n0,v)
-        | n == n0   = Just (attr2str v)
-        | otherwise = Nothing
+        | n == localName n0   = Just (attr2str v)
+        | otherwise           = Nothing
 
 toAttrFrStr   :: String -> String -> Maybe Attribute
-toAttrFrStr n v = Just (n, str2attr v)
+toAttrFrStr n v = Just (N n, str2attr v)
 
 str2attr :: String -> AttValue
 str2attr s =
